// NOTES to check: 
//   DxM s/b nXm NOT mxm
//   neh vs. nsh  (final absorption of PE effects?) (nsols => PEd vs PEm...)

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <math.h>

//#define DEBUG_APPROX_REL ((pp-ped) == TEST_AN)
//#define DEBUG_SIRE_REL ((ss-ped) == TEST_AN)
#define DEBUG_APPROX_REL ((pp-ped) == TEST_AN || (pp-ped) == TEST_DAM || (pp-ped) == TEST_SIRE)
#define DEBUG_SIRE_REL ((ss-ped) == TEST_AN || (ss-ped) == TEST_DAM || (ss-ped) == TEST_SIRE)
#define DEBUG_DAM_REL DEBUG_SIRE_REL

#ifndef DEBUG_APPROX_REL
#define DEBUG_APPROX_REL 0
#define DEBUG_SIRE_REL 0
#define DEBUG_DAM_REL 0
#endif

/************************* GLOBAL VARIABLES ***************************/


/**************************  PROTOTYPES  ******************************/

void flpr_rel( FLOAT *ee, char *cc );
void pr_rel( REAL *ee, char *cc );
void mtrel_abs_D2P( REAL zC );
void mtrel_abs_S2P( REAL zC );
void mtrel_abs_DS2P(); 
void mtrel_abs_MATES();
void mtrel_abs_PARENTS();


//**********************************************************************
/***************************  SUBROUTINES  ****************************/
//**********************************************************************

//**********************************************************************
void flpr_rel( FLOAT *ee, char *cc ) {
  // Convert from EDC to REL and print to stdout
  int i;
  float x;
  static REAL *tmp=0, *Ginv;
  if (!tmp) {
    tmp = (REAL *) init( "flpr_rel: tmp", sizeof(REAL), nsh );
    Ginv = (REAL *) init( "flpr_rel: Ginv", sizeof(REAL), nsh );
    for (i=0; i<neh; i++) Ginv[i] = G[i];
    invert( Ginv, neffects );
    for (i=neh; i<nsh; i++) Ginv[i] = 0.0;
  }
  for (i=0; i<nsh; i++) tmp[i] = Ginv[i] + ee[i];
  invert( tmp, nsols );
  printf( "%s: ", cc );
  for (i=0; i<neffects; i++) if (including_trait(i)) {
    if (G[irc(i,i)] > 0) {
      x = 1.0 - tmp[irc(i,i)] / G[irc(i,i)];
    }else {
      x = 0.0;
    }
    printf( "%8.3f", x );
  }
  printf( "\n" );
  return;
}

//**********************************************************************
void pr_rel( REAL *ee, char *cc ) {
  // Convert from EDC to REL and print to stdout
  int i;
  float x;
  static REAL *tmp=0, *Ginv;
  if (!tmp) {
    tmp = (REAL *) init( "pr_rel: tmp", sizeof(REAL), nsh );
    Ginv = (REAL *) init( "pr_rel: Ginv", sizeof(REAL), nsh );
    for (i=0; i<neh; i++) Ginv[i] = G[i];
    invert( Ginv, neffects );
    for (i=neh; i<nsh; i++) Ginv[i] = 0.0;
  }
  for (i=0; i<nsh; i++) tmp[i] = Ginv[i] + ee[i];
  invert( tmp, nsols );
  printf( "%s: ", cc );
  for (i=0; i<neffects; i++) if (including_trait(i)) {
    if (G[irc(i,i)] > 0) {
      x = 1.0 - tmp[irc(i,i)] / G[irc(i,i)];
    }else {
      x = 0.0;
    }
    printf( "%8.3f", x );
  }
  printf( "\n" );
  return;
}

//**********************************************************************
void mtrel_abs_D2P( REAL zC ) {
  // derive dam contribution to progeny (input dam npe3[] and prog me[]=DxM)
  int i, j, k, ii, jj, ij;
  REAL x;
  for (i=0; i<neffects; i++) {
    for (j=0; j<neffects; j++) CC(i,j) = zC * G2[irc(i,j)];
    for (; j<nsols; j++) CC(i,j) = 0.0;
  }
  for (; i<nsols; i++) {
    for (j=0; j<nsols; j++) CC(i,j) = 0.0;
  }
  for (i=0; i<ntraits; i++) {
    for (j=0; j<nmaternal; j++) {
      jj = ntraits + j;
      if (!nGeq0 || ( (G[irc(i,i)] && G[irc(jj,jj)]) )) {
	CC(i,j+ntraits) += pp->me[i*nmaternal+j];   // direct X maternal  (row = absorb target = prog)
      }
    }
  }
  if (PEvar) {
    for (ii=0; ii<nPEdir; ii++) {
      i = PEdir_trait[ii];
      for (j=0; j<nmaternal; j++) {
	CC(ii+neffects,j+ntraits) += pp->me[i*nmaternal+j];   // PEdir X maternal  (row = absorb target = prog)
      }
      for (jj=0; jj<nPEmat; jj++) {
	j = PEmat_trait[jj];
	CC(ii+neffects,jj+neffects+nPEdir) += pp->me[i*nmaternal+j];   // PEdir X PEmat  (row = absorb target = prog)
      }
    }
    for (i=0; i<ntraits; i++) {
      for (jj=0; jj<nPEmat; jj++) {
	j = PEmat_trait[jj];
	CC(i,jj+neffects+nPEdir) += pp->me[i*nmaternal+j];   // direct X PEmat  (row = absorb target = prog)
      }
    }
  }
  for (i=0; i<nsh; i++) P[i] = Pdam[i];
#ifdef DEBUG
  if (DEBUG_APPROX_REL) {
    pr( stdout, "P", P, nsh );
    fprintf( stdout, "C:\n" );
    for (i=0; i<nsols; i++) {
      for (j=0; j<nsols; j++) fprintf( stdout, "%7.4f", CC(i,j) );
      fprintf( stdout, "\n" );
    }
  }
#endif
#ifdef SLOW
  invert( P, nsols );
  f_CPC( 0, nsols, 0, nsols );
#else
  f2_CPC( 0, nsols, 0, nsols );
#endif
#ifdef DEBUG
  if (DEBUG_APPROX_REL) {
    pr( stdout, "CPC", CPC, nsh );
  }
#endif
  return;
}  

//**********************************************************************
void mtrel_abs_S2P( REAL zC ) {
  // derive sire contribution to progeny (input sire npe3[])
  int i, j, k, ij;
  REAL x;
  for (i=0; i<neffects; i++) {
    for (j=0; j<neffects; j++) CC(i,j) = zC * G2[irc(i,j)];
    for (; j<nsols; j++) CC(i,j) = 0.0;
  }
  for (; i<nsols; i++) {
    for (j=0; j<nsols; j++) CC(i,j) = 0.0;
  }
  for (i=0; i<nsh; i++) P[i] = Psire[i];
#ifdef SLOW
  invert( P, nsols );
  f_CPC( 0, nsols, 0, nsols );
#else
  f2_CPC( 0, nsols, 0, nsols );
#endif
  return;
}  

//**********************************************************************
void mtrel_abs_DS2P() {
  //derive sireXdam contributions to progeny (called from mtrel_abs_PARENTS)
  int i, j, k, ij, m;
  REAL x;

#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      for (i=0; i<neh; i++) Psire[i] -= G2[i];
      pr_rel( Psire, "Rel(S)|" );
      for (i=0; i<neh; i++) Psire[i] += G2[i];
      for (i=0; i<neh; i++) Pdam[i] -= G2[i];
      pr_rel( Pdam, "Rel(D)|" );
      for (i=0; i<neh; i++) Pdam[i] += G2[i];
    }
#endif

  /*
  pr( stdout, "DS2P Pdam", Pdam, nsh );
  pr( stdout, "DS2P Psire", Psire, nsh );
  printf( "DS2P C:\n" );
  for (i=0; i<nsols; i++) {
    for (j=0; j<nsols; j++) printf( "%8.4f", CC(i,j) );
    printf( "\n" );
  }
  printf( "DS2P B:\n" );
  for (i=0; i<nsols; i++) {
    for (j=0; j<nsols; j++) printf( "%8.4f", BB(i,j) );
    printf( "\n" );
  }
  */
  // absorb dam into sire
  for (i=0; i<nsh; i++) P[i] = Pdam[i];
#ifdef SLOW
  invert( P, nsols );
  f_CPC( 0, nsols, 0, nsols );
#else
  f2_CPC( 0, nsols, 0, nsols );
#endif
  /*
  fprintf( stdout, "Dam into Sire:\n" );
  fprintf( stdout, "C: " );
  for (i=0; i<neff2; i++) fprintf( stdout, "%8.4f", C[i] );
  fprintf( stdout, "\n" );
  fprintf( stdout, "B: " );
  for (i=0; i<neff2; i++) fprintf( stdout, "%8.4f", B[i] );
  fprintf( stdout, "\n" );
  pr( stdout, "P: ", P, nsh );
  */
  for (i=0; i<nsh; i++) Psire[i] -= CPC[i];
#ifdef DEBUG
  if (DEBUG_APPROX_REL) {
    printf( "Rel(S) 2 abs C" );
    for (i=0; i<8; i++) printf( " %f", CC(i,i) );
    printf( "\n" );
    printf( "Rel(S) 2 abs CPC" );
    for (i=0; i<8; i++) printf( " %f", CPC[i] );
    printf( "\n" );
  }
#endif

#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      pr_rel( Psire, "Rel(S)|D" );
    }
#endif

  // absorb dam into sire X progeny
#ifdef SLOW
  f_CPB( 0, nsols, 0, nsols, 0, nsols, 0, nsols );
#else
  f2_CPB( 0, nsols, 0, nsols, 0, nsols, 0, nsols );
#endif
  for (i=0; i<nsols; i++) {
    for (j=0; j<nsols; j++) {
      CCsp(i,j) = -G2[irc(i,j)] - CCPB(i,j);
    }
  }

  // absorb dam into progeny
  for (i=0; i<nsols; i++) {
    for (j=0; j<nsols; j++) {
      CC(i,j) = BB(i,j);
    }
  }
#ifdef SLOW
  f_CPC( 0, nsols, 0, nsols );
#else
  f2_CPC( 0, nsols, 0, nsols );
#endif
  for (i=0; i<nsh; i++) Pprog[i] -= CPC[i];

#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      pr_rel( Pprog, "Rel(A)|D" );
    }
#endif

  /*
  printf( "Dam absorbed:\n" );
  pr( stdout, "  Psire", Psire, nsh );
  for (i=0; i<neh; i++) Pprog[i] += G2[i];
  pr( stdout, "  Pprog", Pprog, nsh );
  for (i=0; i<neh; i++) Pprog[i] -= G2[i];
  pr( stdout, "  CCsp ", Csp, neff2*2 );
  */
  // absorb sire into progeny
  for (i=0; i<nsols; i++) {
    for (j=0; j<nsols; j++) CC(j,i) = CCsp(i,j);  // change from sire to progeny target
  }
  for (i=0; i<nsh; i++) P[i] = Psire[i];
#ifdef SLOW
  invert( P, nsols );
  f_CPC( 0, nsols, 0, nsols );
#else
  f2_CPC( 0, nsols, 0, nsols );
#endif
  for (i=0; i<nsh; i++) Pprog[i] -= CPC[i];

#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      pr_rel( Pprog, "Rel(A)|D,S" );
    }
#endif

  return;
}

//**********************************************************************
void mtrel_abs_MATES() {
  // for each parent, loop through mates
  // absorb progeny into parent X mate, then mate into parent
  int i, j, k, ij, m, p, s, nm, v[5];
  REAL x;
  char M2_tempfile[BUFSIZ], M2S_tempfile[BUFSIZ];
  FILE *fM2;

  // store lists of progeny with both parents known and ne > 0, for each sire and dam
  // N.B. ASSUMPTION: prog should still be NULL from original zinit()
  for (pp=ped+1; pp<=ped+nanml; pp++) {
    if (pp->sire && pp->dam) {
      for (x=i=0; !x && i<neffects; i++) x += pp->ne[irc(i,i)];
      if (x) {
	ss = ped + pp->sire;
	dd = ped + pp->dam;
	ss->nprog++;
	dd->nprog++;
	if (dd->sex == MALE) {
	  //sprintf( msg, "Dam %d of Progeny %d is also a Sire !!\n", pp->dam, (int) (pp-ped) );
	  //warning( msg );
	  pp->dam = -pp->dam;
	}
	if (ss->sex == FEMALE) {
	  //sprintf( msg, "Sire %d of Progeny %d is also a Dam !!\n", pp->sire, (int) (pp-ped) );
	  //warning( msg );
	  pp->sire = -pp->sire;
	}
      }
    }
  }
  for (ss=ped+1; ss<=ped+nanml; ss++) {
    if (ss->nprog) {
      if (!pstart && ss->nprog > max_mates) max_mates = ss->nprog;
      ss->prog = zinit( "ss->prog", sizeof( PED *), ss->nprog );
      ss->nprog = 0;
    }
  }
  if (!pstart) {
    max_mates++;
    fprintf( stdout, "Allocating RAM for a maximum of %d mates per sire\n", max_mates );
    pstart = (int *) zinit( "pstart", sizeof(int), max_mates );
  }
  for (p=1; p<=nanml; p++) {
    pp = ped + parents_first[p];
    if (pp->sire > 0 && pp->dam > 0) {
      for (x=i=0; !x && i<neffects; i++) x += pp->ne[irc(i,i)];
      if (x) {
	ss = ped + pp->sire;
	dd = ped + pp->dam;
	ss->prog[ ss->nprog ] = pp;
	ss->nprog++;
	dd->prog[ dd->nprog ] = pp;
	dd->nprog++;
      }
    }
    if (pp->sire < 0) pp->sire = -pp->sire;
    if (pp->dam < 0) pp->dam = -pp->dam;
  }

  if (advanced_mate_absorb) {

    // NOTE: in this section ss/sire could be sire or dam, and dd/dam is the other parent
    /*
    printf( "G2 = " );
    for (i=0; i<nsh; i++) printf( "%11.7f", G2[i] ); printf( "\n" );
    for (i=1; i<=nanml; i++) {
      printf("npe %2d", i );
      for (j=0; j<neh; j++) printf( "%11.7f", ped[i].npe[j] ); printf( "\n" );
    }
    */
    // absorb dams into sires
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      if (!(s%100000)) {
	fprintf( stdout, "%9d) %9d (Sire)\n", s, (int) (ss-ped) );
      }
      if (ss->nprog) {
	if (!ss->npe2) {
	  ss->npe2 = (FLOAT *) zinit( "ss->npe2", sizeof(FLOAT), nsh ); 
	  for (i=0; i<nsh; i++) ss->npe2[i] = ss->npe[i];
	}
	if (ss->sex == MALE) {
	  qsort( (void *)ss->prog, (size_t)ss->nprog, sizeof(PED *), qsort_sire );
	  // count number of mates and progeny start points for each sire
	  nm = k = 0;
	  for (i=0; i<ss->nprog; i++) {
	    if (ss->prog[i]->dam != k) {
	      k = ss->prog[i]->dam;
	      if (nm < max_mates-1) pstart[nm] = i;
	      nm++;
	    }
	  }
	  if (nm < max_mates) {
	    pstart[nm] = ss->nprog;
	  }else {
	    fprintf( stderr, "Sire %d has more than maximum number of mates ... %d > %d\n",
		     parents_first[s], nm, max_mates-1 );
	    exit(9);
	  }
	  ss->mates = init( "ss->mates", sizeof( int ), nm );
	  ss->Xm = init( "ss->Xm", sizeof( FLOAT * ), nm );
	  ss->nmates = nm;
	  // process one mate at a time
	  for (m=0; m<nm; m++) {
	    dd = ped + (ss->mates[m] = ss->prog[pstart[m]]->dam);
	    // absorb all full-sib progeny into sire X mate
	    ss->Xm[m] = zinit( "ss->Xm[]", sizeof( FLOAT ), nsols*nsols );
	    for (p=pstart[m]; p<pstart[m+1]; p++) {
	      pp = ss->prog[p];
	      mtedc_abs_P2SD();
	      for (i=0; i<nsols; i++) {
		for (j=0; j<nsols; j++) {
		  // CPB is dam X sire, so add transpose to sire X dam
		  ss->Xm[m][j*nsols+i] += CCPB(i,j);
		}
	      }
	    }
	    for (i=0; i<nsols; i++) {
	      for (j=0; j<nsols; j++) CCdm(i,j) = ss->Xm[m][i*nsols+j];
	    }
	    // absorb mate
	    mtedc_abs_D2S();
	    /*
	    printf( "absorbing mate %5d into sire %5d  npe2 -= ", dd-ped, ss-ped );
	    for (i=0; i<neh; i++) printf( "%11.7f", CPC[i] ); printf( "\n" );
	    */
	    for (i=0; i<nsh; i++) ss->npe2[i] -= CPC[i];
	  }
#ifdef DEBUG
	  if (FULL_DEBUG) {
	    if (ss->npe2 && ss->npe2 != ss->npe) {
	      fprintf( flog, "sire %d npe2 after absorbing all mates:\n", (int) (ss-ped) );
	      for (i=0; i<nsols; i++) {
		for (j=0; j<=i; j++) fprintf( flog, "%7.4f", ss->npe2[irc(i,j)] );
		fprintf( flog, "\n" );
	      }
	    }
	  }
#endif
	}
      }
    }
    
    // absorb sires into dams
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      if (!(s%100000)) {
	fprintf( stdout, "%9d) %9d (Dam)\n", s, (int) (ss-ped) );
      }
      if (ss->nprog) {
	if (!ss->npe2) {
	  ss->npe2 = (FLOAT *) zinit( "ss->npe2", sizeof(FLOAT), nsh ); 
	  for (i=0; i<nsh; i++) ss->npe2[i] = ss->npe[i];
	}
	if (ss->sex == FEMALE) {
	  qsort( (void *)ss->prog, (size_t)ss->nprog, sizeof(PED *), qsort_dam );
	  // count number of mates and progeny start points for each dam
	  nm = k = 0;
	  for (i=0; i<ss->nprog; i++) {
	    if (ss->prog[i]->sire != k) {
	      k = ss->prog[i]->sire;
	      if (nm < max_mates-1) pstart[nm] = i;
	      nm++;
	    }
	  }
	  if (nm < max_mates) {
	    pstart[nm] = ss->nprog;
	  }else {
	    fprintf( stderr, "Dam %d has more than maximum number of mates ... %d > %d\n",
		     parents_first[s], nm, max_mates-1 );
	    exit(9);
	  }
	  ss->mates = init( "ss->mates", sizeof( int ), nm );
	  ss->Xm = init( "ss->Xm", sizeof( FLOAT * ), nm );
	  ss->nmates = nm;
	  // process one mate at a time
	  for (m=0; m<nm; m++) {
	    dd = ped + (ss->mates[m] = ss->prog[pstart[m]]->sire);
	    for (k=0; k<dd->nmates && (ss-ped) != dd->mates[k]; k++);
	    if (k<dd->nmates) {
	      ss->Xm[m] = dd->Xm[k];
	    }else {
	      sprintf( msg, "dam %d not found in mate list for sire %d\n",
		       (int) (ss-ped), (int) (dd-ped) );
	      error( msg );
	    }
	    // absorb all full-sib progeny into dam X mate
	    // take transpose of SxD for DxS
	    for (i=0; i<nsols; i++) {
	      for (j=0; j<nsols; j++) CCdm(i,j) = ss->Xm[m][j*nsols+i];
	    }
	    // absorb mate
	    mtedc_abs_D2S();
	    /*
	    printf( "absorbing mate %5d into dam %5d  npe2 -= ", dd-ped, (int) (ss-ped) );
	    for (i=0; i<neh; i++) printf( "%11.7f", CPC[i] ); printf( "\n" );
	    */
	    for (i=0; i<nsh; i++) ss->npe2[i] -= CPC[i];
	  }
#ifdef DEBUG
	  if (FULL_DEBUG) {
	    if (ss->npe2 && ss->npe2 != ss->npe) {
	      fprintf( flog, "dam %d npe2 after absorbing all mates:\n", (int) (ss-ped) );
	      for (i=0; i<nsols; i++) {
		for (j=0; j<=i; j++) fprintf( flog, "%7.4f", ss->npe2[irc(i,j)] );
		fprintf( flog, "\n" );
	      }
	    }
	  }
#endif
	}
      }
    }
    /*
    printf( "G2 = " );
    for (i=0; i<nsh; i++) printf( "%11.7f", G2[i] ); printf( "\n" );
    for (i=1; i<=nanml; i++) {
      printf("npe %2d", i );
      for (j=0; j<neh; j++) printf( "%11.7f", ped[i].npe[j] ); printf( "\n" );
    }
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      printf( "%d has %d mates: ", ss-ped, ss->nmates );
      for (i=0; i<ss->nmates; i++) printf( "%d ", ss->mates[i] ); printf( "\n" );
    }
    */

    // use npe3 to absorb sires into sires and dams into dams, given npe2 and Xm
    // copy npe2 to npe3
    for (p=1; p<=nanml; p++) {
      pp = ped + parents_first[p];
      if (pp->npe2) {
	pp->npe3 = (FLOAT *) init( "pp->npe3", sizeof(FLOAT), nsh ); 
	for (i=0; i<nsh; i++) pp->npe3[i] = pp->npe2[i];
      }else {
	if (meyer_absorb) pp->npe = pp->ne;
	pp->npe2 = pp->npe;
	pp->npe3 = pp->npe2;
      }
    }

    sprintf( M2_tempfile, "crEDC_pid%d_M2", getpid() );
    sprintf( M2S_tempfile, "crEDC_pid%d_M2S", getpid() );
    fM2 = fop( M2_tempfile, "wb" );
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      if (!(s%100000)) {
	fprintf( stdout, "%9d) %9d (Sire|Dam)\n", s, (int) (ss-ped) );
      }
      if (ss->nmates > 1) {
	//	printf( "%d has %d mates: ", ss-ped, ss->nmates );
	//	for (i=0; i<ss->nmates; i++) printf( "%d ", ss->mates[i] ); printf( "\n" );
	// write mate1 mate2 in_list1 in_list2 for sire/dam
	for (m=0; m<ss->nmates; m++) {
	  i = ss->mates[m];
	  for (j=m+1; j<ss->nmates; j++) {
	    k = ss->mates[j];
	    if (i<k) {
	      v[0] = i;
	      v[1] = k;
	      v[3] = m;
	      v[4] = j;
	    }else {
	      v[0] = k;
	      v[1] = i;
	      v[3] = j;
	      v[4] = m;
	    }
	    v[2] = ss - ped;
	    //	    printf( "v=%d,%d,%d,%d,%d\n", v[0], v[1], v[2], v[3], v[4] );
	    fwrite( v, sizeof(int), 5, fM2 );
	  }
	}
      }
    }
    fclose( fM2 );
    sprintf( msg, "/g1/programs/pete/bin/runGE/falcon/memsort -r%d -ka1ia%di -o %s %s ",
	     sizeof(int)*5, 1+sizeof(int), M2S_tempfile, M2_tempfile );
    system( msg );
    fM2 = fop( M2S_tempfile, "rb" );
    
    m = p = 0;
    for (i=0; i<nsols; i++) {
      for (j=0; j<nsols; j++) CCdm(i,j) = 0.0;
    }
    s = 1;
    while( s ) {
      s = fread( v, sizeof(int), 5, fM2 );
      //      for (i=0; i<5; i++) printf( "%9d", v[i] ); printf( "\n" );
      if (!s || (v[0] != m || v[1] != p)) {
	// co-absorb the pair of sires (OR dams)
	// e.g. dam[i] X dam[i'] absorption:  npe3[i] -= CPC  (C=CCdm  P=npe2[i'])
	if (m && p) {
	  // p into m
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) CC(i,j) = CCdm(i,j);
	  }
	  for (i=0; i<nsh; i++) P[i] = G2[i] + ped[p].npe2[i];
#ifdef SLOW
	  invert( P, nsols );
	  f_CPC( 0, nsols, 0, nsols );
#else
	  /*
	  printf( "  C:\n" );
	  for (i=0; i<neffects; i++) {
	    for (j=0; j<neffects; j++) printf( "%11.7f", CC(i,j) ); printf( "\n" );
	  }
	  pr( stdout, "P", P, nsh );
	  */
	  f2_CPC( 0, nsols, 0, nsols );
#endif
	  /*
	  printf( "npe3 %5d: ", m );
	  for (i=0; i<neh; i++) printf( "%11.7f", ped[m].npe3[i] ); printf( "\n" );
	  printf( "          - " );
	  for (i=0; i<neh; i++) printf( "%11.7f", CPC[i] ); printf( "\n" );
	  */
	  for (i=0; i<nsh; i++) ped[m].npe3[i] -= CPC[i];
	  /*
	  printf( "          = " );
	  for (i=0; i<neh; i++) printf( "%11.7f", ped[m].npe3[i] ); printf( "\n" );
	  */

	  // m into p
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) CC(i,j) = CCdm(j,i);
	  }
	  for (i=0; i<nsh; i++) P[i] = G2[i] + ped[m].npe2[i];
#ifdef SLOW
	  invert( P, nsols );
	  f_CPC( 0, nsols, 0, nsols );
#else
	  /*
	  printf( "  C:\n" );
	  for (i=0; i<neffects; i++) {
	    for (j=0; j<neffects; j++) printf( "%11.7f", CC(i,j) ); printf( "\n" );
	  }
	  pr( stdout, "P", P, nsh );
	  */
	  f2_CPC( 0, nsols, 0, nsols );
#endif

	  /*
	  printf( "npe3 %5d: ", p );
	  for (i=0; i<neh; i++) printf( "%11.7f", ped[p].npe3[i] ); printf( "\n" );
	  printf( "          - " );
	  for (i=0; i<neh; i++) printf( "%11.7f", CPC[i] ); printf( "\n" );
	  */
	  for (i=0; i<nsh; i++) ped[p].npe3[i] -= CPC[i];
	  /*
	  printf( "          = " );
	  for (i=0; i<neh; i++) printf( "%11.7f", ped[p].npe3[i] ); printf( "\n" );
	  printf( "\n" );
	  */
	}
	m = v[0];
	p = v[1];
	for (i=0; i<nsols; i++) {
	  for (j=0; j<nsols; j++) CCdm(i,j) = 0.0;
	}
      }
      if (s) {
	// accumulate current sireXsire (OR damXdam)
	// CCdm += CPB (C=Xm[v[3]] B=Xm[v[4]] P=ped[v[2]].npe2)
	ss = ped + v[2];
	if (ss->sex == MALE) {
	  // transpose for target row = Dam instead of Sire
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) CC(i,j) = ss->Xm[v[3]][j*nsols+i];
	    for (j=0; j<nsols; j++) BB(i,j) = ss->Xm[v[4]][j*nsols+i];
	  }
	}else {
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) CC(i,j) = ss->Xm[v[3]][i*nsols+j];
	    for (j=0; j<nsols; j++) BB(i,j) = ss->Xm[v[4]][i*nsols+j];
	  }
	}
	for (i=0; i<nsh; i++) P[i] = G2[i] + ss->npe[i];
#ifdef SLOW
	invert( P, nsols );
	f_CPB( 0, nsols, 0, nsols, 0, nsols, 0, nsols );
#else
	/*
	printf( "  C:\n" );
	for (i=0; i<neffects; i++) {
	  for (j=0; j<neffects; j++) printf( "%11.7f", CC(i,j) ); printf( "\n" );
	}
	printf( "  B:\n" );
	for (i=0; i<neffects; i++) {
	  for (j=0; j<neffects; j++) printf( "%11.7f", BB(i,j) ); printf( "\n" );
	}
	pr( stdout, "P", P, nsh );
	*/
	f2_CPB( 0, nsols, 0, nsols, 0, nsols, 0, nsols );
#endif
	for (i=0; i<nsols; i++) {
	  // CCdm is block m(=v[0]) x p(=v[1])
	  for (j=0; j<nsols; j++) CCdm(i,j) += CCPB(i,j);  // negative of Dam X Dam block
	}
	/*
	printf( "  Cdm:\n" );
	for (i=0; i<neffects; i++) {
	  for (j=0; j<neffects; j++) printf( "%11.7f", CCdm(i,j) ); printf( "\n" );
	}
	*/
      }
    }

    sprintf( msg, "rm %s %s", M2_tempfile, M2S_tempfile );
    system( msg );

  }else {

    // NOTE: in this section ss/sire could be sire or dam, and dd/dam is the other parent

// absorb dams into sires
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      if (!(s%100000)) {
	fprintf( stdout, "%9d) %9d (Sire)\n", s, (int) (ss-ped) );
      }
      if (ss->nprog) {
	if (!ss->npe2) {
	  ss->npe2 = (FLOAT *) zinit( "ss->npe2", sizeof(FLOAT), nsh ); 
	  for (i=0; i<nsh; i++) ss->npe2[i] = ss->npe[i];
	}
	if (ss->sex == MALE) {
	  qsort( (void *)ss->prog, (size_t)ss->nprog, sizeof(PED *), qsort_sire );
	  // count number of mates and progeny start points for each sire
	  nm = k = 0;
	  for (i=0; i<ss->nprog; i++) {
	    if (ss->prog[i]->dam != k) {
	      k = ss->prog[i]->dam;
	      if (nm < max_mates-1) pstart[nm] = i;
	      nm++;
	    }
	  }
	  if (nm < max_mates) {
	    pstart[nm] = ss->nprog;
	  }else {
	    fprintf( stderr, "Sire %d has more than maximum number of mates ... %d > %d\n",
		     parents_first[s], nm, max_mates-1 );
	    exit(9);
	  }
	  // process one mate at a time
	  for (m=0; m<nm; m++) {
	    dd = ped + ss->prog[pstart[m]]->dam;
	    // absorb all full-sib progeny into sire X mate
	    for (i=0; i<nsols; i++) {
	      for (j=0; j<nsols; j++) CCdm(i,j) = 0.0;
	    }
	    for (p=pstart[m]; p<pstart[m+1]; p++) {
	      pp = ss->prog[p];
	      mtedc_abs_P2SD();
	      for (i=0; i<nsols; i++) {
		for (j=0; j<nsols; j++) {
		  // CPB is dam X sire, so add transpose to sire X dam
		  CCdm(j,i) += CCPB(i,j);
		}
	      }
	    }
	    // absorb mate
	    mtedc_abs_D2S();
	    for (i=0; i<nsh; i++) ss->npe2[i] -= CPC[i];
#ifdef DEBUG
	    if (DEBUG_SIRE_REL) {
	      printf( "Rel(S) 1 abs C", (int) (dd-ped) );
	      for (i=0; i<8; i++) printf( "%8.4f", CC(i,i) );
	      printf( "\n" );
	      printf( "Rel(S) 1 abs CPC", (int) (dd-ped) );
	      for (i=0; i<8; i++) printf( "%8.4f", CPC[i] );
	      printf( "\n" );
	    }
#endif
	  }
#ifdef DEBUG
	  if (DEBUG_SIRE_REL) {
	    if (ss->npe2 && ss->npe2 != ss->npe) {
	      sprintf( msg, "sire %d npe2 after absorbing all mates (%d prog)",
		       (int) (ss-ped), ss->nprog );
	      flpr( stdout, msg, ss->npe2, nsh );
	    }
	  }
#endif
	}
      }
    }
    
    // absorb sires into dams
    for (s=1; s<=nanml; s++) {
      ss = ped + parents_first[s];
      if (!(s%100000)) {
	fprintf( stdout, "%9d) %9d (Dam)\n", s, (int) (ss-ped) );
      }
      if (ss->nprog) {
	if (!ss->npe2) {
	  ss->npe2 = (FLOAT *) zinit( "ss->npe2", sizeof(FLOAT), nsh ); 
	  for (i=0; i<nsh; i++) ss->npe2[i] = ss->npe[i];
	}
	if (ss->sex == FEMALE) {
	  qsort( (void *)ss->prog, (size_t)ss->nprog, sizeof(PED *), qsort_dam );
	  // count number of mates and progeny start points for each dam
	  nm = k = 0;
	  for (i=0; i<ss->nprog; i++) {
	    if (ss->prog[i]->sire != k) {
	      k = ss->prog[i]->sire;
	      if (nm < max_mates-1) pstart[nm] = i;
	      nm++;
	    }
	  }
	  if (nm < max_mates) {
	    pstart[nm] = ss->nprog;
	  }else {
	    fprintf( stderr, "Dam %d has more than maximum number of mates ... %d > %d\n",
		     parents_first[s], nm, max_mates-1 );
	    exit(9);
	  }
	  // process one mate at a time
	  for (m=0; m<nm; m++) {
	    dd = ped + ss->prog[pstart[m]]->sire;
	    // absorb all full-sib progeny into dam X mate
	    for (i=0; i<nsols; i++) {
	      for (j=0; j<nsols; j++) CCdm(i,j) = 0.0;
	    }
	    for (p=pstart[m]; p<pstart[m+1]; p++) {
	      pp = ss->prog[p];
	      mtedc_abs_P2SD();
	      for (i=0; i<nsols; i++) {
		for (j=0; j<nsols; j++) {
		  // CPB is dam X sire, correct since dam is the absorb target
		  CCdm(i,j) += CCPB(i,j);
		}
	      }
#ifdef DEBUG
	if (DEBUG_DAM_REL) {
	  printf( "Rel(S2D) p=%d (%d)  Cdm", p, (int) (pp-ped) );
	  for (i=0; i<8; i++) printf( "%8.4f", CCdm(i,i) );
	  printf( "\n" );
	}
#endif
	    }
	    // absorb mate
	    mtedc_abs_D2S();
	    for (i=0; i<nsh; i++) ss->npe2[i] -= CPC[i];
	  }
#ifdef DEBUG
	  if (DEBUG_DAM_REL) {
	    if (ss->npe2 && ss->npe2 != ss->npe) {
	      sprintf( msg, "dam %d npe2 before absorbing all mates into dams (%d prog)",
		       (int) (ss-ped), ss->nprog );
	      flpr( stdout, msg, ss->npe2, nsh );
	    }
	  }
#endif
	}
      }
    }

    // copy npe2 to npe3
    for (p=1; p<=nanml; p++) {
      pp = ped + parents_first[p];
      if (pp->npe2) {
	pp->npe3 = (FLOAT *) init( "pp->npe3", sizeof(FLOAT), nsh ); 
	for (i=0; i<nsh; i++) pp->npe3[i] = pp->npe2[i];
      }else {
	if (meyer_absorb) pp->npe = pp->ne;
	pp->npe2 = pp->npe;
	pp->npe3 = pp->npe2;
      }
    }

  }

  return;
}

//**********************************************************************
void mtrel_abs_PARENTS() {
  // de-absorb progeny then absorb parent
  //   (input sire/dam npe3,npe2,npe[] prog npe2[])
  int i, j, k, ij, m, p, s, neAN, neDAM;
  REAL x, zAA, zAS, zAD, zSS, zDD, zSD;
  PED *ppp;

  if (meyer_absorb) {
    for (p=1; p<=nanml; p++) {
      pp = ped + parents_first[p];
      pp->npe3 = pp->npe2 = pp->npe = pp->ne;
    }
  }

  for (p=1; p<=nanml; p++) {
    pp = ped + parents_first[p];
    if (!(p%100000)) {
      fprintf( stdout, "%9d) %9d%9d%9d\n", p, (int) (pp-ped), pp->sire, pp->dam );
    }
#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      printf( "%d) PARENTS of %d: %d %d\n", p, (int) (pp-ped), pp->sire, pp->dam );
      pr( stdout, "G-inverse", G2, neh );
    }
#endif
    if (pp->matdam && pp->matdam != pp->dam) {
      // absorb recipient cow into ET calf
      dd = ped + pp->matdam;
      // de-absorb this progeny
      for (i=0; i<nsh; i++) Pdam[i] = dd->ne[i];
      mtedc_abs_PR();
      for (i=0; i<nsh; i++) Pdam[i] += CPC[i];
      mtedc_abs_RP();
      for (i=0; i<nsh; i++) pp->ne[i] -= CPC[i];
      if (pp->npe) for (i=0; i<nsh; i++) pp->npe[i] -= CPC[i];
      // replace DxM with zero co-incidence for the genetic dam
      for (i=0; i<ntraits; i++) {
	for (j=0; j<nmaternal; j++) pp->me[i*nmaternal+j] = 0.0;
      }
#ifdef DEBUG
      if (DEBUG_APPROX_REL) {
	sprintf( msg, " %d) ne After Recip absorbed", (int) (pp-ped) );
	flpr( stdout, msg, pp->ne, nsh );
	if (pp->npe) {
	  sprintf( msg, " %d) npe After Recip absorbed", (int) (pp-ped) );
	  flpr( stdout, msg, pp->npe, nsh );
	  flpr_rel( pp->npe, "Rel(A)|D" );
	}
      }
#endif
    }
    if (pp->sire || pp->dam) {

      ss = ped + pp->sire;
      dd = ped + pp->dam;
      neAN = neDAM = 0;
      i = 0;
      while( !neAN && i<nsols ) {
	neAN = (pp->ne[irc(i,i)] > 0);
	i++;
      }
      if (neAN) {
	i = 0;
	while( !neDAM && i<nmaternal ) {
	  neDAM = (pp->me[i*nmaternal+i] > 0);
	  i++;
	}
      }

      if (pp->sire && pp->dam && !meyer_absorb) {
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  flpr_rel( ss->npe, "Rel(S)" );
	  flpr_rel( dd->npe, "Rel(D)" );
	  flpr_rel( pp->npe, "Rel(A)" );
	  flpr_rel( ss->npe2, "Rel(S2)" );
	  flpr_rel( dd->npe2, "Rel(D2)" );
	}
#endif
	zAA = 2.0;
	zAS = zAD = -1.0;
	zSS = zDD = zSD = 0.5;
	for (i=0; i<nsh; i++) Psire[i] = G2[i] + ss->npe2[i];
	for (i=0; i<nsh; i++) Pdam[i] = G2[i] + dd->npe2[i];
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	  pr( stdout, "Pdam", Pdam, nsh );
	  printf( "de-absorb mates\n" );
	}
#endif
	// de-absorb mates
	for (i=0; i<nsols; i++) {
	  for (j=0; j<nsols; j++) {
	    CCdm(i,j) = 0.0;
	  }
	}
	ppp = pp;
	for (m=0; m<dd->nprog; m++) {
	  pp = dd->prog[m];
	  if (pp->sire == ppp->sire) {
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	  printf( "Rel(S) m=%d (%d) Cdm", m, (int) (pp-ped) );
	  for (i=0; i<8; i++) printf( " %f", CCdm(i,i) );
	  printf( "\n" );
	}
#endif
	    mtedc_abs_P2SD();
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	}
#endif
	    for (i=0; i<nsols; i++) {
	      for (j=0; j<nsols; j++) {
		CCdm(i,j) += CCPB(i,j);
	      }
	    }
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	  printf( "Rel(S) m=%d (%d) Cdm", m, (int) (pp-ped) );
	  for (i=0; i<8; i++) printf( " %f", CCdm(i,i) );
	  printf( "\n" );
	}
#endif
	  }
	}
	pp = ppp;
	// ... sire from dam
	for (i=0; i<nsols; i++) {
	  for (j=0; j<nsols; j++) {
	    CC(i,j) = CCdm(i,j);
	  }
	}
	for (i=0; i<nsh; i++) P[i] = G2[i] + ss->npe[i];
#ifdef SLOW
	invert( P, nsols );
	f_CPC( 0, nsols, 0, nsols );
#else
	f2_CPC( 0, nsols, 0, nsols );
#endif
	for (i=0; i<nsh; i++) Pdam[i] += CPC[i];

	// ... dam from sire
	for (i=0; i<nsols; i++) {
	  for (j=0; j<nsols; j++) {
	    // transpose dm to md
	    CC(j,i) = CCdm(i,j);
	  }
	}
	for (i=0; i<nsh; i++) P[i] = G2[i] + dd->npe[i];
#ifdef SLOW
	invert( P, nsols );
	f_CPC( 0, nsols, 0, nsols );
#else
	f2_CPC( 0, nsols, 0, nsols );
#endif
	for (i=0; i<nsh; i++) Psire[i] += CPC[i];
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	  printf( "Rel(S) de-abs C" );
	  for (i=0; i<8; i++) printf( "%8.4f", CC(i,i) );
	  printf( "\n" );
	  printf( "Rel(S) de-abs CPC" );
	  for (i=0; i<8; i++) printf( "%8.4f", CPC[i] );
	  printf( "\n" );
	}
#endif
	
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Psire", Psire, nsh );
	  pr( stdout, "Pdam", Pdam, nsh );
	  for (i=0; i<neh; i++) Psire[i] -= G2[i];
	  pr_rel( Psire, "Rel(S):D" );
	  for (i=0; i<neh; i++) Psire[i] += G2[i];
	  for (i=0; i<neh; i++) Pdam[i] -= G2[i];
	  pr_rel( Pdam, "Rel(D):S" );
	  for (i=0; i<neh; i++) Pdam[i] += G2[i];
	  printf( "deabsorb progeny\n" );
	}
#endif
	for (i=0; i<neh; i++) {
	  Psire[i] += zSS * G2[i];
	  Pdam[i] += zDD * G2[i];
	}
	if (neAN) {
	  // de-absorb progeny from sire
	  mtedc_abs_P2S( zAA, zAS, zSS );
	  for (i=0; i<nsh; i++) Psire[i] -= CPC[i];// - zSS * G2[i];
	  
	  // de-absorb progeny from dam  (same contribution as sire unless DxM)
	  if (neDAM) {
	    mtedc_abs_P2D( zAA, zAD, zDD );
	  }
	  for (i=0; i<nsh; i++) Pdam[i] -= CPC[i];// - zDD * G2[i];
	  //	  for (i=0; i<nsh; i++) Pdam[i] += CPC[i];
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    pr( stdout, "Psire", Psire, nsh );
	    pr( stdout, "Pdam", Pdam, nsh );
	  }
#endif
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) {
	      BB(j,i) = CC(i,j);  // transpose to change absorb target from dam to progeny
	    }
	  }
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) CC(i,j) = zSD * G2[irc(i,j)];
	  }
	}else {
	  for (i=0; i<nsols; i++) {
	    for (j=0; j<nsols; j++) {
	      BB(i,j) = zAD * G2[ irc(i,j) ];
	      CC(i,j) = zSD * G2[ irc(i,j) ];
	    }
	  }
	}

	// absorb dam and sire into progeny  (BB=progXdam, CC=sireXdam)
	for (i=0; i<nsh; i++) Pprog[i] = G2[i] + pp->npe2[i];

#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Pprog", Pprog, nsh );
	}
#endif
	mtrel_abs_DS2P();
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  pr( stdout, "Pprog", Pprog, nsh );
	}
#endif

	for (i=0; i<nsh; i++) {
	  //	  pp->npe2[i] += Pprog[i] - pp->npe[i];
	  //	  pp->npe[i] = Pprog[i];

	  //pp->npe3[i] += Pprog[i] - pp->npe2[i];
	  pp->npe3[i] = Pprog[i];
	}
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  flpr_rel( pp->npe3, "Rel(A)" );
	}
#endif
      }else {
#ifdef DEBUG
	if (DEBUG_APPROX_REL) {
	  flpr_rel( pp->npe, "Rel(A)" );
	}
#endif
	if (pp->sire) {
	  // de-absorb from sire, dam is unknown
	  zAA = 4.0/3.0;
	  zAS = -2.0/3.0;
	  zSS = 1.0/3.0;
	  for (i=0; i<nsh; i++) Psire[i] = (4.0/3.0) * G2[i] + ss->npe3[i];
	  if (neAN) {
	    mtedc_abs_P2S( 4.0/3.0, -2.0/3.0, 1.0/3.0 );
	    for (i=0; i<nsh; i++) Psire[i] -= CPC[i];
	  }
	  // absorb sire into progeny
	  mtrel_abs_S2P( -2.0/3.0 );
	  for (i=0; i<nsh; i++) pp->npe3[i] += (1.0/3.0) * G2[i] - CPC[i];
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    flpr_rel( pp->npe, "Rel(A)|S" );
	  }
#endif
	}
	if (pp->dam) {
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    if (pp->npe) {
	      sprintf( msg, " %d) npe Before absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	    }
	  }
#endif
	  // de-absorb from dam, sire is unknown
	  if (sire_model) {
	    zAA = 16.0/15.0;
	    zAD = -4.0/15.0;
	    zDD = 1.0/15.0;
	  }else {
	    zAA = 4.0/3.0;
	    zAD = -2.0/3.0;
	    zDD = 1.0/3.0;
	  }
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    if (pp->npe) {
	      sprintf( msg, " %d) npe ...1 absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	    }
	  }
#endif
	  for (i=0; i<nsh; i++) Pdam[i] = zAA * G2[i] + dd->npe3[i];
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    if (pp->npe) {
	      sprintf( msg, " %d) npe ...2 absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	    }
	  }
#endif
	  if (neAN) {
	    mtedc_abs_P2D( zAA, zAD, zDD );
	  }else {
	    mtedc_abs_P2S( zAA, zAD, zDD );
	  }
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    if (pp->npe) {
	      sprintf( msg, " %d) npe ...3 absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	    }
	  }
#endif
	  for (i=0; i<nsh; i++) Pdam[i] -= CPC[i];
	  // absorb dam into progeny
	  mtrel_abs_D2P( zAD );
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    if (pp->npe) {
	      sprintf( msg, " %d) npe ...4 absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	    }
	  }
#endif
	  // Note for below that: zDD = (zAA - 1.0)
	  for (i=0; i<nsh; i++) pp->npe3[i] += zDD * G2[i] - CPC[i];
#ifdef DEBUG
	  if (DEBUG_APPROX_REL) {
	    printf( "Adj to npe3:\n" );
	    for (i=k=0; i<nsols; i++) {
	      for (j=0; j<=i; j++,k++) printf( "%9.4f", zDD*G2[k] - CPC[k] );
	      printf( "\n" );
	    }
	    if (pp->npe) {
	      sprintf( msg, " %d) npe After absorbing the dam", (int) (pp-ped) );
	      flpr( stdout, msg, pp->npe, nsh );
	      flpr_rel( pp->npe, "Rel(A)|D" );
	    }
	  }
#endif
	}
      }
    }
#ifdef DEBUG
    if (DEBUG_APPROX_REL) {
      sprintf( msg, " %d) ne After Parents absorbed", (int) (pp-ped) );
      flpr( stdout, msg, pp->ne, nsh );
      if (pp->npe) {
	sprintf( msg, " %d) npe After Parents absorbed", (int) (pp-ped) );
	flpr( stdout, msg, pp->npe, nsh );
      }
    }
#endif
    //########## mimic old pgm before adding npe3
    if (1 || !advanced_mate_absorb) {
      if (pp->npe3 != pp->npe2) for (i=0; i<nsh; i++) pp->npe2[i] = pp->npe3[i];
    }
    //##########
  }
  // move npe to medc for sires
  for (ss=ped+1; ss<=ped+nanml; ss++) {
    if (ss->npe3 != ss->npe) for (i=0; i<nsh; i++) ss->npe[i] = ss->npe3[i];
    if (ss->medc != NULL) {
      for (i=0; i<neh; i++) ss->medc[i] = ss->npe3[i];
    }
  }
  
  return;
}

